/*
 * Miau
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define F_CPU 16000000

int count = 0;
int servo1_value = 200;

int max(int a, int b){
	return (a > b) ? a : b;
}

int min(int a, int b){
	return (a < b) ? a : b;
}

void init_servo1() {
	/* set output pin for servo 1 */
	DDRC |= (1 << PC4);

	/* scriu 1 pe pinul PC4 */
	PORTC |= (1 << PC4);

	/* CTC cu TOP la OCR0A, prescaler 64 */
	TCCR0A = (1 << WGM01);
	TCCR0B = (3 << CS00);

	/* counter-ul setat la 0 si TOP-ul la 1*/
	TCNT0 = 0;
	OCR0A = 1;

	/* enable timer compare interrupt */
	TIMSK0 |= (1 << OCIE0A);
}

/*
* Initializeaza canalele A si B ale timer-ului 1
* seteaza pinii de iesire pentru servo 2 si 3
*/
void init_servos23(){
	/* set output pin for OC1A <-> servo 2 */
	DDRD |= (1 << PD5);
	/* set output pin for OC1B <-> servo 3*/
	DDRD |= (1 << PD4);

	/* FastPWM cu TOP la ICR1, prescaler de 8
	*  clear on comp, set on bottom 
	*/
	TCCR1A |= 1<<WGM11 | 1<<COM1A1 | 1<<COM1A0 | 1<<COM1B1 | 1<<COM1B0;
	TCCR1B |= 1<<WGM13 | 1<<WGM12 | 1<<CS11;

	/*
	* Perioada de 50Hz <=> 20 ms 
	* TOP = F_CPU/50*PS - 1
	*/
	ICR1 = 39999;

	OCR1A = ICR1 - 2000;
	OCR1B = ICR1 - 4000;
}

/*
* primeste unghiul in grade intre 0 si 180
* roteste servo-motorul 2
*/
void rotate_servo2(int angle) {
	angle = max(angle, 0);
	angle = min(angle , 180);

	float ms = angle / 180 + 1;
	OCR1A = ICR1 - (ms * 2000);
}

/*
* primeste unghiul in grade intre 0 si 180
* roteste servo-motorul 3
*/
void rotate_servo3(int angle) {
	angle = max(angle, 0);
	angle = min(angle , 180);

	float ms = angle / 180 + 1;
	OCR1B = ICR1 - (ms * 2000);
}

ISR(TIMER0_COMPA_vect) {
	count++;

	if(count == servo1_value)	
		/* pun pinul PC4 pe 0 */
		PORTC &= ~(1 << PC4);
		
	if(count == 4999) {
		/* resetez counter-ul si pun pinul PC4 pe 1 */
		PORTC |= (1 << PC4);
		count = 0;
	}
}

void miau() {
	/* initializez cele 3 servo */
	init_servo1();
	init_servos23();

	/* Setez pinii butoanelor ca intrare. */
	DDRD &= ~(1 << PD6);
	DDRB &= ~(1 << PB4);
	DDRB &= ~(1 << PB5);

	/* Activez rezistenta de pull-up pentru pinii butoanelor. */
	PORTD |= (1 << PD6);
	PORTB |= (1 << PB4);
	PORTB |= (1 << PB5);

	/* Setez pinul PD7 ca iesire. */
	DDRD |= (1 << PD7);

	/* Sting LED-ul. */
	PORTD &= ~(1 << PD7);

	/* move to painting area servo 1*/
	servo1_value = 200;
	_delay_ms(100);

	while (1){
		int t = 0;
		/* if painting button is pressed */
		if ((PINB & (1 << PB4)) == 0) {
			/* Aprind LED-ul. */
			PORTD |= (1 << PD7);

			/* move arm down */
			OCR1A = ICR1 - 2650;
			_delay_ms(500);

			/* paint */
			for(t = 0; t < 4; t++){
				/* move brush up */
				OCR1B = ICR1 - 3600;
				_delay_ms(100);
				/* move brush down */
				OCR1B = ICR1 - 4000;
				_delay_ms(100);
			}

			/* move arm up */
			OCR1A = ICR1 - 2000;
			_delay_ms(500);
			
		}
		/* if dipping button is pressed */	
		else if ((PINB & (1 << PB5)) == 0) {
			/* Sting LED-ul. */
			PORTD &= ~(1 << PD7);

			/* move to diping area servo 1*/
			OCR1A = ICR1 - 2000;
			servo1_value = 270;
			_delay_ms(100);

			/* dip */
			_delay_ms(500);
			OCR1A = ICR1 - 2800;
			_delay_ms(500);
			OCR1A = ICR1 - 2000;
			_delay_ms(500);

			/* move back to painting area servo 1 */
			servo1_value = 200;
			_delay_ms(100);
		}	
		
	}
}

int main(void) {
	sei();

	miau();

	while(1);
	return 0;
}
